#include "mcdp.h"

/**
 * Copyright (C) 2001-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * author:         Tino Reichardt <milky-mcdp@mcmilk.de>
 * status:         works for now
 *
 * details:
 * - for linux scsi generic interface
 *
 * todo:
 * - some simple checking for errors ?
 */

#if WANT_LINUX_SCSI_GENERIC
int cmd_lx_sg(scmd_t *cmd)
{
	struct sg_io_hdr io;
	int r;
#if DEBUG_CDB
	char *file="LX_sg.log";
#endif

	byte_zero(&io, sizeof(io));

	io.interface_id = 'S';
	io.flags    |= SG_FLAG_DIRECT_IO;
	io.timeout   = 1000 * MCDP_SCSI_TIMEOUT;
	io.cmdp      = (unsigned char*)cmd->cmd.s;
	io.cmd_len   = cmd->cmdlen;
	io.dxferp    = (unsigned char*)cmd->buf.s;
	io.dxfer_len = cmd->buflen;
	io.sbp       = (unsigned char*)cmd->sense.s;
	io.mx_sb_len = MCDP_MAX_SENSE;

	if isset(cmd->flags, D_NONE) {
		set(io.dxfer_direction, SG_DXFER_NONE);
	} else {
		set(io.dxfer_direction, SG_DXFER_TO_FROM_DEV);
	}

#if DEBUG_CDB
	cmd_debug(file, CDB_BEFORE);
#endif

	r=ioctl(cd->fd, SG_IO, &io);
	byte_copy(cmd->sense.s, MCDP_MAX_SENSE, io.sbp);

#if DEBUG_CDB
	cmd_debug(file, CDB_AFTER);
#endif

	return r;
}
#endif /* WANT_LINUX_SCSI_GENERIC */


#if 0

#define SG_DXFER_NONE -1        /* e.g. a SCSI Test Unit Ready command */
#define SG_DXFER_TO_DEV -2      /* e.g. a SCSI WRITE command */
#define SG_DXFER_FROM_DEV -3    /* e.g. a SCSI READ command */
#define SG_DXFER_TO_FROM_DEV -4 /* treated like SG_DXFER_FROM_DEV with ... */

typedef struct sg_io_hdr
{
  int interface_id;           /* [i] 'S' for SCSI generic (required) */
  int dxfer_direction;        /* [i] data transfer direction  */
  unsigned char cmd_len;      /* [i] SCSI command length ( <= 16 bytes) */
  unsigned char mx_sb_len;    /* [i] max length to write to sbp */
  unsigned short int iovec_count; /* [i] 0 implies no scatter gather */
  unsigned int dxfer_len;     /* [i] byte count of data transfer */
  void * dxferp;              /* [i], [*io] points to data transfer memory or scatter gather list */
  unsigned char * cmdp;       /* [i], [*i] points to command to perform */
  unsigned char * sbp;        /* [i], [*o] points to sense_buffer memory */
  unsigned int timeout;       /* [i] MAX_UINT->no timeout (unit: millisec) */
  unsigned int flags;         /* [i] 0 -> default, see SG_FLAG... */
  int pack_id;                /* [i->o] unused internally (normally) */
  void * usr_ptr;             /* [i->o] unused internally */
  unsigned char status;       /* [o] scsi status */
  unsigned char masked_status;/* [o] shifted, masked scsi status */
  unsigned char msg_status;   /* [o] messaging level data (optional) */
  unsigned char sb_len_wr;    /* [o] byte count actually written to sbp */
  unsigned short int host_status; /* [o] errors from host adapter */
  unsigned short int driver_status;/* [o] errors from software driver */
  int resid;                  /* [o] dxfer_len - actual_transferred */
  unsigned int duration;      /* [o] time taken by cmd (unit: millisec) */
  unsigned int info;          /* [o] auxiliary information */
} sg_io_hdr_t;
#endif
