#include "mcdp.h"
#include "runcmd.h"
#include "getkey.h"
#include "shuffle.h"

/**
 * Copyright (C) 2001-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

static void mcdp_doit_now(char *ch)
{
	stralloc sa, keys, right;

	stralloc_init(&sa);
	stralloc_init(&keys);
	stralloc_init(&right);

	switch (*ch) {

	/*****************************/
	/* group 1, playing commands */
	/*****************************/

	case '\033':
		/* cursor keys */
		if (str_len(ch) != 3) break;
		switch (ch[2]) {
			case 'A':
				cd->cd_seek(+TIME_FW_FAST);
				break; /* c_up */
			case 'B':
				cd->cd_seek(-TIME_RW_FAST);
				break; /* c_down */
			case 'C':
				cd->cd_seek(+TIME_FW_SLOW);
				break; /* c_left */
			case 'D':
				cd->cd_seek(-TIME_RW_SLOW);
				break; /* c_right */
		}
		break;

	/* play next title */
	case KEY_TRACK_NEXT:
		cd->cd_play(+1);
		set(cd->updates, U_MAIN);
		break;

	/* play prev title */
	case KEY_TRACK_PREV:
		cd->cd_play(-1);
		set(cd->updates, U_MAIN);
		break;

	/* trackmodifier + */
	case KEY_TRACKMOD_NEXT:
		if (cd->trackmod < 9) cd->trackmod++; else cd->trackmod=0;
		set(cd->updates, U_TRACKMOD);
		break;

	/* trackmodifier - */
	case KEY_TRACKMOD_PREV:
		if (cd->trackmod > 0) cd->trackmod--; else cd->trackmod=9;
		set(cd->updates, U_TRACKMOD);
		break;

	/* start / stop playing */
	case KEY_PLAY_STOP:
		if (cd->as == AS_PLAY) {
			cd->cd_stop();
		} else {
			cd->cd_play(0);
			set(cd->updates, U_MAIN);
		}
		break;

	/* pause / resume playing */
	case KEY_PAUSE_RESUME:
		cd->cd_pauseresume();
		break;

	/* redraw screen */
	case CTRL_X('l'):
		set(cd->updates, U_ALL);
		break;

	/* jump directly to some track */
	case '0':
		if (!cd->trackmod && !((int)ch-'0')) break;
		/* XXX else fall into next ... */

	case '1': case '2': case '3':
	case '4': case '5': case '6':
	case '7': case '8': case '9':
		cd->current=((cd->trackmod*10)+((int)ch-'0'));
		cd->cd_play(cd->current);
		set(cd->updates, U_MAIN);
		break;

	/****************************/
	/* group 2, playing options */
	/****************************/

	/* shuffle */
	case KEY_SHUFFLE:
		toogle(cd->flags, F_M_SHUFFLE);
		shuffle();
		set(cd->updates, U_METHOD|U_MAIN);
		break;

	/* repeat */
	case KEY_REPEAT:
		if (cd->repeat < R_OFF) {
			cd->repeat++;
		} else {
			cd->repeat=R_CDROM;
		}
		set(cd->updates, U_REPEAT);
		break;

	/* method */
	case KEY_METHOD_PLUS:
		if (cd->method >= 2) cd->method=0;
		else cd->method += 1;
		set(cd->updates, U_METHOD);
		break;

	case KEY_METHOD_MINUS:
		if (cd->method <= 0) cd->method=2;
		else cd->method -= 1;
		set(cd->updates, U_METHOD);
		break;

	/* intro time */
	case KEY_INTRO_PLUS:
		if (cd->intro < 30) {
			cd->intro++;
			set(cd->updates, U_METHOD);
		}
		break;

	case KEY_INTRO_MINUS:
		if (cd->intro > 3)  {
			cd->intro--;
			set(cd->updates, U_METHOD);
		}
		break;

	/* volume */
	case KEY_VOL_PLUS:
		if (cd->volume >= 250) cd->volume=255;
		else cd->volume += 5;
		cd_volume(cd->volume);
		set(cd->updates, U_VOLUME);
		break;

	case KEY_VOL_MINUS:
		if (cd->volume <= 5) cd->volume=0;
		else cd->volume -= 5;
		cd_volume(cd->volume);
		set(cd->updates, U_VOLUME);
		break;


	/************************************/
	/* group 3, track database handling */
	/************************************/

	/* clear track space */
	case CTRL_X('f'):
		db_free();
		set(cd->updates, U_MAIN);
		break;

	case KEY_EDIT:
		show_main_reset();
		show_sa(&c_question);
		if (!stralloc_copys(&sa, "")) die_nomem();
		if (!stralloc_copys(&keys, lang_msg(MSG_EDIT_KEYS))) die_nomem();
		if (!stralloc_cats(&keys, lang_msg(MSG_ABORT_KEY))) die_nomem();
		if (!stralloc_0(&keys)) die_nomem();
		if (!stralloc_copys(&right, lang_msg(MSG_EDIT_QUESTION))) die_nomem();
		show_main(&sa, &right);
		show_sa(&c_note);
		if (!stralloc_copys(&right, lang_msg(MSG_EDIT_CDDB))) die_nomem();
		show_main(&sa, &right);
		if (!stralloc_copys(&right, lang_msg(MSG_EDIT_WMDB))) die_nomem();
		show_main(&sa, &right);
		if (!stralloc_copys(&right, lang_msg(MSG_ABORT))) die_nomem();
		show_main(&sa, &right);

		flush();
		switch (getkey(keys.s)) {
		case 0:
			if (!cd->cddbfile.s) break; /* can't happen */

			if (!stralloc_copys(&sa, mcdp_env_editor))
				die_nomem();
			if (!stralloc_cats(&sa, " ")) die_nomem();
			if (!stralloc_cats(&sa, cd->cddbfile.s))
				die_nomem();
			sa.s[sa.len]=0;

			/* create a new cddb file, if there is no one */
			if (db_cddb() == -1) db_cddb_write();
	
			/* run the editor */
			runcmd(sa.s);

			/* re-read database */
			db_cddb();
	
			/* update all things */
			set(cd->updates, U_ALL);
			break;
		case 1:
			if (!mcdp_env_workmandb) break; /* can't happen */

			if (!stralloc_copys(&sa, mcdp_env_editor))
				die_nomem();
			if (!stralloc_cats(&sa, " ")) die_nomem();
			if (!stralloc_cats(&sa, mcdp_env_workmandb))
				die_nomem();
			sa.s[sa.len]=0;

			/* run the editor */
			runcmd(sa.s);
			
			/* re-read database */
			db_wmdb();

			/* update all things */
			set(cd->updates, U_ALL);
			break;
		case 2:
			set(cd->updates, U_MAIN);
			break;
		} /* end of switch() */
		break;

	case KEY_READ:
		show_main_reset();
		show_sa(&c_question);
		if (!stralloc_copys(&sa, "")) die_nomem();
		if (!stralloc_copys(&keys, lang_msg(MSG_READ_KEYS))) die_nomem();
		if (!stralloc_cats(&keys, lang_msg(MSG_ABORT_KEY))) die_nomem();
		if (!stralloc_0(&keys)) die_nomem();
		if (!stralloc_copys(&right, lang_msg(MSG_READ_QUESTION)))
			die_nomem();
		show_main(&sa, &right);
		show_sa(&c_note);
		if (!stralloc_copys(&right, lang_msg(MSG_READ_CDTEXT))) die_nomem();
		show_main(&sa, &right);
		if (!stralloc_copys(&right, lang_msg(MSG_READ_CDDB))) die_nomem();
		show_main(&sa, &right);
		if (!stralloc_copys(&right, lang_msg(MSG_READ_WMDB))) die_nomem();
		show_main(&sa, &right);
		if (!stralloc_copys(&right, lang_msg(MSG_READ_CDDBP))) die_nomem();
		show_main(&sa, &right);
		if (!stralloc_copys(&right, lang_msg(MSG_READ_HTTP))) die_nomem();
		show_main(&sa, &right);
		if (!stralloc_copys(&right, lang_msg(MSG_ABORT))) die_nomem();
		show_main(&sa, &right);

		flush();
		switch (getkey(keys.s)) {
			case 0:
				db_free();
				db_cdtext();
				set(cd->updates, U_MAIN);
				break;
			case 1:
				db_free();
				db_cddb();
				set(cd->updates, U_MAIN);
				break;
			case 2:
				db_free();
				db_wmdb();
				set(cd->updates, U_MAIN);
				break;
			case 3:
				db_free();
				db_cddbp_read();
				db_cddb();
				set(cd->updates, U_MAIN);
				break;
			case 4:
				db_free();
				db_http_read();
				db_cddb();
				set(cd->updates, U_MAIN);
				break;
			case 5:
				set(cd->updates, U_MAIN);
				break;
		}
		break;

	case KEY_SAVE:
		show_main_reset();
		show_sa(&c_question);
		if (!stralloc_copys(&sa, "")) die_nomem();
		if (!stralloc_copys(&keys, lang_msg(MSG_SAVE_KEYS))) die_nomem();
		if (!stralloc_cats(&keys, lang_msg(MSG_ABORT_KEY))) die_nomem();
		if (!stralloc_0(&keys)) die_nomem();
		if (!stralloc_copys(&right, lang_msg(MSG_SAVE_QUESTION))) die_nomem();
		show_main(&sa, &right);
		show_sa(&c_note);
		if (!stralloc_copys(&right, lang_msg(MSG_SAVE_CDDB))) die_nomem();
		show_main(&sa, &right);
		if (!stralloc_copys(&right, lang_msg(MSG_SAVE_CDDBP))) die_nomem();
		show_main(&sa, &right);
		if (!stralloc_copys(&right, lang_msg(MSG_SAVE_HTTP))) die_nomem();
		show_main(&sa, &right);
		if (!stralloc_copys(&right, lang_msg(MSG_SAVE_EMAIL))) die_nomem();
		show_main(&sa, &right);
		if (!stralloc_copys(&right, lang_msg(MSG_ABORT))) die_nomem();
		show_main(&sa, &right);

		flush();
		switch (getkey(keys.s)) {
			case 0:
				db_cddb_write();
				set(cd->updates, U_MAIN);
				break;
			case 1:
				db_cddbp_write();
				set(cd->updates, U_MAIN);
				break;
			case 2:
				db_http_write();
				set(cd->updates, U_MAIN);
				break;
			case 3:
				db_email_write();
				set(cd->updates, U_MAIN);
				break;
			case 4:
				set(cd->updates, U_MAIN);
				break;
		}
		break;


	/****************************/
	/* group 4, title list keys */
	/****************************/

	case KEY_LIST_UP:
		show_main_scroll(-1);
		break;

	case KEY_LIST_DOWN:
		show_main_scroll(+1);
		break;


	/***************************/
	/* group 5, all other keys */
	/***************************/

	case KEY_MIXER:
		if (!stralloc_copys(&sa, mcdp_env_mixer)) die_nomem();
		sa.s[sa.len]=0;
		runcmd(sa.s); /* run the mixer */

		/* update all, after closing the mixer */
		set(cd->updates, U_ALL);
		break;

	case KEY_EJECT:
		cd->xs=XS_OPENING;
		show_status();
		cd->cd_eject();
		cd->xs=XS_NONE;
		show_status();
		break;

	case KEY_CLOSE:
		cd->xs=XS_CLOSING;
		show_status();
		cd->cd_close();
		cd->xs=XS_NONE;
		show_status();
		break;

	case KEY_DEVICE:
		if (cd->device < 2) cd->device++;
		else cd->device=0;
		set(cd->updates, U_DEVICE);
		break;

	case '?':
	case 'h':
		runcmd("man mcdp");
		set(cd->updates, U_ALL);
		break;

	case KEY_QUIT:
		mcdp_quit(0);
		break;

	} /* switch */

	stralloc_free(&sa);
	stralloc_free(&keys);
	stralloc_free(&right);

	return;
}

void mcdp_doit(void)
{
	tai6464 t;
	char x[23];
	int r;

	set(cd->updates, U_ALL);
	io_nonblock(0); /* set input to nonblocking */
	for (;;) {
		/* update all thing, which have changed */
		mcdp_update(); flush();

		taia_now(&t);
		taia_addsec(&t, &t, 1);
		io_wantread(0);

		/* wait one second, for some input */
		io_waituntil(t);
		r = io_canread();
		io_dontwantread(0);

		/* fd = 0 is ready for reading
		 * XXX: use io_tryread() here!
		 */
		if (r == 0) {
			byte_zero(x, sizeof(x));
			r = buffer_get(buffer_0, x, 10);
			if (*x) mcdp_doit_now(x);
		}
	}

	return;
}
