#ifndef M_PARAMS_H
#define M_PARAMS_H

/**
 * Copyright (C) 2001-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * Please edit these settings to match your system.
 * When you're done, type 'make' to compile and link.
 * Then you may install with 'make install'.
 */

/* if $CDROM isn't set */
#define DEFAULT_DEVCDROM   "/dev/cdrom"

/* if $MIXER isn't set */
#define DEFAULT_MIXER      "mmix"

/* if $EDITOR isn't set */
#define DEFAULT_EDITOR     "vim"

/* if $LANG isn't set */
#define DEFAULT_LANG       "en"

/* the default server variables */
#define DEFAULT_CDDB_SERVER    "freedb.freedb.org:8880"
#define DEFAULT_CDDB_WEBSERVER "http://freedb.org/~cddb/cddb.cgi"
#define DEFAULT_CDDB_SUBMIT    "http://freedb.org/~cddb/submit.cgi"

/* if there is no port in $CDDB_SERVER, we take this: */
#define DEFAULT_CDDBPORT    8880

/* if there is no port in $HTTP_PROXY, we take this: */
#define DEFAULT_PROXYPORT   3128

/* intro times */
#define MCDP_INTRO_DEFAULT   7 /* default intro time in seconds */
#define MCDP_INTRO_MIN       3
#define MCDP_INTRO_MAX      30

#define MCDP_PATH_MAX      200 /* max path+file length for this app! */
#define MCDP_CDAUDIO_MAX    99 /* max tracks of a audio cd */
#define MCDP_PLIST_MAX      10 /* max playlists */
#define MCDP_SCSI_TIMEOUT   10 /* ... */
#define MCDP_CDDB_TIMEOUT    9 /* seconds, to wait for a response from serva */
#define MCDP_CREATMODE    0644 /* used, for new entries @CDDB_PATH/xyxyxyxy */

#define TIME_FW_FAST        30 /* FF (seconds for fast forward) */
#define TIME_FW_SLOW         6 /* FW (seconds for forward) */
#define TIME_RW_FAST        30 /* FR (seconds for fast rewind) */
#define TIME_RW_SLOW         6 /* RW (seconds for rewind) */

/* playing commands */
#define KEY_TRACKMOD_NEXT  '+' /* trackmodifier +1 */
#define KEY_TRACKMOD_PREV  '-' /* trackmodifier -1 */
#define KEY_PLAY_STOP      's' /* start/stop playing full cd/playlist */
#define KEY_PAUSE_RESUME   'p' /* pause/resume playing audio-cd */
#define KEY_TRACK_NEXT     ' ' /* next title */
#define KEY_TRACK_PREV     'b' /* previous title */

/* playing options */
#define KEY_SHUFFLE        '*' /* shuffle: on, off */
#define KEY_REPEAT         'r' /* repeating: cdrom, track, off */
#define KEY_METHOD_PLUS    'm' /* playing method: normal / intro / playlist */
#define KEY_METHOD_MINUS   'M' /* playing method: normal / intro / playlist */
#define KEY_INTRO_PLUS     'i' /* increase time for intro */
#define KEY_INTRO_MINUS    'I' /* decrease time for intro */
#define KEY_VOL_PLUS       '>' /* increase cdrom volume */
#define KEY_VOL_MINUS      '<' /* decrease cdrom volume */

/* track database handling */
#define KEY_READ           'G' /* edit title information */
#define KEY_EDIT           'E' /* edit title information */
#define KEY_SAVE           'S' /* save title information */

/* general keys */
#define KEY_MIXER          'a' /* start audio mixer command */
#define KEY_EJECT          'o' /* open cdrom */
#define KEY_CLOSE          'c' /* close cdrom */
#define KEY_QUIT           'q' /* quit mcdp */
#define KEY_DEVICE         'd' /* switch between device name/file/driver info */

/* title list keys */
#define KEY_LIST_UP        '(' /* title list -=5 */
#define KEY_LIST_DOWN      ')' /* title list +=5 */

/*
 * ESC seqences, you may change some of them with environment variables
 *
 * style: 00=none 01=bold 04=underscore 05=blink 07=reverse 08=concealed
 * fg:    30=black 31=red 32=green 33=yellow 34=blue 35=magenta 36=cyan 37=grey
 * bg:    40=black 41=red 42=green 43=yellow 44=blue 45=magenta 46=cyan 47=grey
 */

#define C_BOX         "\e[0;34m"  /* blue   - boxdrawing */
#define C_NORMAL      "\e[0;37m"  /* grey   - normal/default/tracks */
#define C_STATUS      "\e[0;31m"  /* red    - status lines / current track */
#define C_DESCRIPTION "\e[1;33m"  /* yellow - descriptions */
#define C_HEADLINE    "\e[0;33m"  /* brown  - version, headlines */
#define C_CDNAME      "\e[1;33m"  /* yellow */
#define C_TRACKS      "\e[0m"     /* normal - tracks!=current */
#define C_CTRACK      "\e[1;37m"  /* white  - current track */
#define C_NOTE        "\e[0m"     /* additional infos, also used by getkey() */
#define C_QUESTION    "\e[1;34m"  /* additional infos, also used by getkey() */
#define C_CDDB_SEND   "\e[0;34m"  /* response from cddb server */
#define C_CDDB_RETR   "\e[0;36m"  /* command from mcdp to the server */
#define C_CDDB_FAIL   "\e[1;31m"  /* failures */

/*
 * if you have some ugly old terminals, set the define to 0 !!!
 */
#if 1

#define BX_IN      "\016"
#define BX_OUT     "\017"
#define AC_TL      "l"   /* top left */
#define AC_TR      "k"   /* top right */
#define AC_BL      "m"   /* bottom left */
#define AC_BR      "j"   /* bottom right */
#define AC_MT      "v"   /* top 2middle */
#define AC_MB      "w"   /* bottom 2middle */
#define AC_ML      "t"   /* left 2middle */
#define AC_MR      "u"   /* right 2middle */
#define AC_MM      "n"   /* middle + */
#define AC_HL      "q"   /* hline - */
#define AC_VL      "x"   /* vline | */

#else

#define BX_IN      ""
#define BX_OUT     ""
#define AC_TL      ","   /* top left */
#define AC_TR      "."   /* top right */
#define AC_BL      "`"   /* bottom left */
#define AC_BR      "'"   /* bottom right */
#define AC_MT      "^"   /* top 2middle */
#define AC_MB      "v"   /* bottom 2middle */
#define AC_ML      ">"   /* left 2middle */
#define AC_MR      "<"   /* right 2middle */
#define AC_MM      "+"   /* middle + */
#define AC_HL      "-"   /* hline - */
#define AC_VL      "|"   /* vline | */

#endif

/*
 * scsi command sending functions
 */

#if defined(__linux__)
#define WANT_LINUX_SCSI_GENERIC   1
#define WANT_LINUX_SEND_PACKET    1
#endif

#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
#define WANT_BSD_SCSIIO           1
#endif

#if defined(__hpux__)
#define WANT_HPUX_SCTL            1
#endif

#if defined(__sun__)
#define WANT_SUN_USCSI            1
#endif

#endif
