#include "show.h"
#include "slash.h"

/**
 * Copyright (C) 2001-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/**
 * show_tracks_nl - show "name / extd" data of the tracks
 *
 * @param l      length of the color definitions in the string
 * @param left   is shown in the left main window (one time)
 * @param r      length of the color definitions in the string
 * @param right  for the right side, can have multiple lines
 * @return       nothing
 */
static void show_tracks_nl(int l, stralloc *left, int r, stralloc *right)
{
	stralloc sa;
	char *s, *t;
	int i;

	/* replace all '\n' with '\0' and show them later as extra lines */
	stralloc_init(&sa);

	if (!stralloc_copy(&sa, right)) die_nomem();
	sa.s[sa.len]=0;

	for (s=t=sa.s; *t; t++, s++) {
		if (*t=='\\') {
			if (*++t) {
				if (*t == 'n') { sa.len--; *s=0; }
			}
		} else *s=*t; /* no '\n' */
	} *s=0; /* terminate new end */

	for (i=0, s=sa.s; i < (int)sa.len; i+=str_len(s+i)+1) {
		if (i) {
			if (!stralloc_copys(left, "")) die_nomem();
			l=0;
		}
		if (!stralloc_copys(right, sa.s+i)) die_nomem();
		show_main_color(l, left, r, right);
	}

	stralloc_free(&sa);

	return;
}

static void show_track(int track, int real)
{
	stralloc left, right;
	int l=0, r=0;

	stralloc_init(&left);
	stralloc_init(&right);

	if (cd->current == track) {
		if (!stralloc_cat(&left, &c_ctrack)) die_nomem();
		l=c_ctrack.len;
	} else {
		if (!stralloc_cat(&left, &c_tracks)) die_nomem();
		l=c_tracks.len;
	}

//  if ((i-offset)==(maxy-9)) { break; }
//  m=((i-(pluskey+1)*10)<0 && (i-(pluskey+1)*10)>-11); /* is shortcut? */
//
//  gotoxy(3,9+i-offset);
//  if (m&&(i==cd->title[0])) _printf(C_STATUS "%02d" C_NORMAL, i); /* current */
//  else if (m||(i==cd->title[0])) _printf(T_BOLD "%02d" C_NORMAL, i); /* current */

	if (!stralloc_cat_u32pw(&left, track, 2, '0')) die_nomem();
	if (!stralloc_cats(&left, " ")) die_nomem();

	if (!stralloc_cat_u32pw(&left, real, 2, '0')) die_nomem();
	if (!stralloc_cats(&left, cd->t[real].audio?"d ":"a ")) die_nomem();

	if (!stralloc_cat_lba(&left, cd->t[real].len)) die_nomem();
#if 0
	stralloc_cats(&right, "line: ");
	stralloc_cat_u32pw(&right, line,  2, ' ');
	stralloc_cats(&right, " track: ");
	stralloc_cat_u32pw(&right, track, 2, ' ');
	stralloc_cats(&right, " real: ");
	stralloc_cat_u32pw(&right, real,  2, ' ');
	stralloc_cats(&right, " lba: ");
	stralloc_cat_u32pw(&right, cd->t[track].lba,  6, ' ');
	stralloc_cats(&right, " len: ");
	stralloc_cat_u32pw(&right, cd->t[track].len,  5, ' ');
	stralloc_cats(&right, " cddb: ");
	stralloc_cat_u32pw(&right, cd->t[track].cddb, 6, ' ');
	stralloc_cats(&right, "  ");
#endif
	if (cd->t[real].name.len) {
		if (!stralloc_cats(&right, cd->t[real].name.s)) die_nomem();
	}

	if (cd->t[real].name.len && cd->t[real].extd.len) {
		if (!stralloc_cats(&right, slash)) die_nomem();
	}

	if (cd->t[real].extd.len) {
		if (!stralloc_cats(&right, cd->t[real].extd.s)) die_nomem();
	}


	show_tracks_nl(l, &left, r, &right);
	stralloc_free(&left);
	stralloc_free(&right);

	return;
}

void show_tracks(void)
{
	stralloc left, right;
	int i;

	stralloc_init(&left);
	stralloc_init(&right);

	/* clear old stuff */
	show_main_reset();
	show_main_clear();

	/* default color */
	show_sa(&c_tracks);

	/* 1. name / extd of cdrom */
	if (!stralloc_cat(&right, &c_cdname)) die_nomem();
	if (!stralloc_cat(&right, &cd->t[0].name)) die_nomem();
	if (cd->t[0].extd.len && cd->t[0].name.len) {
		if (!stralloc_cats(&right, slash)) die_nomem();
	}
	if (!stralloc_cat(&right, &cd->t[0].extd)) die_nomem();

	if (!cd->t[0].extd.len && !cd->t[0].name.len) {
		/* right side one down ...  */
		if (!stralloc_cats(&right, "\n")) die_nomem();
	}

	i = c_cdname.len;
	show_tracks_nl(0, &left, i, &right);
	stralloc_free(&left);
	stralloc_free(&right);

	/* 2. name / extd of track */
	#if 0
	for (i=1+cd->offset;i<=cd->titles && i<=99 && 8+i<show_maxy-2+cd->offset; i++) {
		if (isset(cd->flags, F_M_SHUFFLE)) {
			show_track(i, cd->t[i].rnd);
		} else {
			show_track(i+cd->offset, i+cd->offset);
		}
	}
	#endif

	for (i=1; i<=cd->titles; i++) {
		if (isset(cd->flags, F_M_SHUFFLE)) {
			show_track(i, cd->t[i].rnd);
		} else {
			show_track(i, i);
		}
	}

	return;
}
